require_relative 'instantiator/argumentless_instantiator'
require_relative 'instantiator/pass_through_instantiator'
require_relative 'instantiator/provider_instantiator'
require_relative 'instantiator/singleton_instantiator'
require_relative 'instantiator/singleton_provider_instantiator'
require_relative 'resolver'

module CrystalClear
  class RegistrationBuilder

    attr_reader :clazz, :classes

    def initialize(clazz)
      @clazz = clazz
      @classes = [clazz]
      @block = lambda { |add_procedure| add_procedure.call(@classes, ArgumentlessInstantiator.new(@clazz), @clazz) }
    end

    def self.for(clazz)
      RegistrationBuilder.new(clazz)
    end

    def under(*classes)
      arr = classes.respond_to?(:each) ? classes : [classes]
      arr.each { |c| @classes.push(c) }
      @classes.uniq!
      self
    end

    def resolves_to_singleton
      @block = lambda { |add_procedure| add_procedure.call(@classes, SingletonInstantiator.new(@clazz), @clazz) }
      self
    end

    def resolves_to_instance(instance)
      @block = lambda { |add_procedure| add_procedure.call(@classes, PassThroughInstantiator.new(instance), @clazz) }
      self
    end

    def resolves_to_provider(provider_class)
      @block = lambda { |add_procedure| add_procedure.call(@classes, ProviderInstantiator.new(provider_class), @clazz) }
      self
    end

    def resolves_to_singleton_provider(provider_class)
      @block = lambda { |add_procedure| add_procedure.call(@classes, SingletonProviderInstantiator.new(provider_class), @clazz) }
      self
    end

    def build(add_procedure)
      @block.call(add_procedure)
    end

  end
end
